#version 120

#define VOLUMETRIC_CLOUDS
#define DYNAMIC_AMBIENT



#define CLOUD_MODE  // Cloud mode: ON=ADD OFF=MIX
#define CLOUD_MIN 120.0 // Minimum cloud height [60 120 180 240 300]
#define CLOUD_MAX 150.0 // Maximum cloud height [90 150 210 270 330]

#define UNDERWATER_DARKNESS 1.5
#define LAVA_BRIGHTNESS 2.0
#define UNDERWATER_FOG_DENSITY 0.1
#define LAVA_FOG_DENSITY 0.3


const int RGBA8 = 1;
const int RGB16 = 2;
const int RGBA16 = 3;
const int gcolorFormat = RGBA16;
const int gdepthFormat = RGBA8;
const int gnormalFormat = RGB16;

const int shadowMapResolution = 512;   // Shadow map resolution [512 1024 2048 3072 4096]
const int noiseTextureResolution = 64;  // Noise texture size [16 32 64]

const float shadowDistance = 64.0; // Radius of shadow map [64.0 96.0 128.0 160.0]
const float shadowIntervalSize = 1.0;  // Shadow correction interval [1.0 2.0 4.0]
const float	sunPathRotation	= -23.;   // Tilt the sun [-23.4 0 23.4]
const float	ambientOcclusionLevel = 1.0;   // Ambient occlusion level [0.0 0.5 0.8 1.0]
const float drynessHalflife = 600.0;  // Time to wet! 1.0=fast 900.0=slow [1.0 300.0 600.0 900.0]
const float wetnessHalflife = 900.0;  // Time to dry! 1.0=fast 900.0=slow [1.0 300.0 600.0 900.0]

#define SHADOW_MAP_BIAS 0.3    // Shadow Map Bias [0.0 0.3 0.6 0.85]

const vec3 torchcolor = vec3(1.2, 0.39, 0.02);

#define TORCH_POWER 4.0
#define TORCH_INTENSITY 2.0

varying vec4 texcoord;
varying vec4 heldLight;

varying vec3 lightVector;
varying vec3 lightOffset0;
varying vec3 lightOffset3;
varying vec3 sunlight_color;
varying vec3 ambient_color;

varying float transition_fading;
varying float ambient_transition;
varying float gp00;
varying float gp11;
varying float gp22;
varying float gp32;
varying float gp2232;
varying vec2 igp;
varying float flicker;
varying float timeMoon;
varying float timeLightTransition;
varying float timeSun;

uniform sampler2D gcolor;   // buffer 0
uniform sampler2D gdepth;   // specularity - red green blue gloss
uniform sampler2D gnormal;  // buffer 2
uniform sampler2D gaux1;    // buffer 4
uniform sampler2D gaux2;    // rain from gbuffer_weather.fsh
uniform sampler2D depthtex0;
uniform sampler2D depthtex2;
uniform sampler2D shadow;
uniform sampler2D watershadow;
uniform sampler2D shadowcolor;
uniform sampler2D noisetex;

uniform mat4 gbufferModelViewInverse;
uniform mat4 shadowProjection;
uniform mat4 shadowModelView;

uniform vec3 cameraPosition;

uniform float far;
uniform float near;
uniform float rainStrength;
uniform float wetness;
uniform float frameTimeCounter;
uniform float viewWidth;
uniform float viewHeight;

uniform int isEyeInWater;

vec3 proj(in vec3 pos) {
    //    vec4 p = gbufferProjection * vec4(pos, 1.0);
    //    p = p * 0.5 + 0.5;
    //    return p.xyz / p.w;
    return vec3(pos.x * gp00, pos.y * gp11, gp2232) / pos.z + 0.5;
}

vec3 invproj(in vec3 p) {
//    vec4 pos = gbufferProjectionInverse * vec4(p * 2.0 - 1.0, 1.0);
//    return pos.xyz / pos.w;
    vec3 pos = p - 0.5;
    float z = gp32 / (pos.z + gp22);
    return vec3(pos.xy * igp, -1.0) * z;
}

float nspec(in vec3 pos, in vec3 lvector, in vec3 normal) {
    
    // half vector
    vec3 cHalf = normalize(lvector + pos);
    float normalDotHalf = dot(normal, cHalf);
    
    // beckmann's distribution function D
    float normalDotHalf2 = normalDotHalf * normalDotHalf;
    float roughness2 = 0.25;
    float exponent = -(1.0 - normalDotHalf2) / (normalDotHalf2 * roughness2);
    float D = exp(exponent) / (roughness2 * normalDotHalf2 * normalDotHalf2);
	
    // fresnel term F
    float halfDotEye = dot(cHalf, pos);
    float F = clamp(pow(1.0 - halfDotEye, 5.0), 0.0, 1.0);
    
    // self shadowing term G
    float normalDotEye = dot(normal, pos);
    float normalDotLight = dot(normal, lvector);
    float X = 2.0 * normalDotHalf / halfDotEye;
    float G = min(1.0, min(X * normalDotLight, X * normalDotEye));
    const float pi = 3.1415927;
    float CookTorrance = (D * F * G) / (pi * normalDotEye);
	
    return max(CookTorrance / pi, 0.00001);
}

float ld(in float depth) {
    return (2.0 * near) / (far + near - (2.0 * depth - 1.0) * (far - near));
}

float noise(in vec3 x) {
    vec3 p = floor(x);
    vec3 f = fract(x);
    f = f * f * (3.0 - 2.0 * f);
    vec2 uv = (p.xy + vec2(37.0, 17.0) * p.z) + f.xy;
    return mix(texture2D(noisetex, uv/noiseTextureResolution).x, texture2D(noisetex, (uv + vec2(37.0, 17.0))/noiseTextureResolution).x, f.z);
}

float getCloudNoise(in vec3 worldPos) {
    vec3 coord = worldPos;
    coord.x += frameTimeCounter * 3.0;
    coord *= 0.12;
    float n = noise(coord) * 0.5;
    n += noise(coord * 3.0) * 0.35;
    n += noise(coord * 9.0) * 0.125;
    n += noise(coord * 27.0) * 0.0625;
    n += noise(coord * 81.0) * 0.05125;
    
    // Adjust cloud size and opacity based on time of day
    float dayOpacity = 1.0;
    float nightOpacity = 0.6;
    float cloudOpacity = mix(nightOpacity, dayOpacity, 1.0 - timeMoon);
    
    // Adjust for light transition periods (dawn/dusk)
    cloudOpacity = mix(cloudOpacity, (dayOpacity + nightOpacity) * 0.1, timeLightTransition);
    
    // Adjust for rain
    float cloudSize = mix(0.8, 1.2, rainStrength);
    cloudOpacity = mix(cloudOpacity, 1.0, rainStrength * 0.5);
    
    float rain_effect = 0.16 * min(1.25, wetness + rainStrength) - 0.6;
    return max((n + rain_effect) * cloudSize * cloudOpacity, 0.0) / (0.8 + rain_effect);
}


float cloudRayMarching(in vec3 start, in vec3 end) {
    vec3 dir = normalize(end - start);
    float ay = abs(dir.y);
    float cloudMin = mix(CLOUD_MIN, CLOUD_MIN * 0.9, rainStrength);
    float cloudMax = mix(CLOUD_MAX, CLOUD_MAX * 1.1, rainStrength);
    float cutoff = 0.00001;  // Define cutoff here
    
    if (max(start.y, end.y) < cloudMin || cloudMax < min(start.y, end.y) || ay < cutoff) {
        // no clouds
        return 0.0;
    }
    
	
	
    float sum = 0.0;
    vec3 ray = start + (max(0.0, CLOUD_MIN - start.y) - max(0.0, start.y - CLOUD_MAX)) * dir / dir.y;
    vec3 step = dir / ay;
    float wall = mix(max(CLOUD_MIN, end.y), min(CLOUD_MAX, end.y), step.y * 0.5 + 0.5);
    float dest = min(ray.y * step.y + 24.0, wall * step.y) * step.y;
    float dy = abs(dest - ray.y);
    for (int i = 0; i < dy; i++) {
        ray += step;
        sum += getCloudNoise(ray) * 0.04;
    }
    return sum * min(1.0, (ay - cutoff) * 5);
	
	float dayIntensity = 1.0;
    float nightIntensity = 0.7;
    float cloudIntensity = mix(nightIntensity, dayIntensity, 1.0 - timeMoon);
    cloudIntensity = mix(cloudIntensity, (dayIntensity + nightIntensity) * 0.5, timeLightTransition);
    
    return sum * min(1.0, (ay - cutoff) * 5) * cloudIntensity;
}

//////////////////////////////main//////////////////////////////

void main() {
    vec3 color = texture2D(gcolor, texcoord.st).rgb;
    float spec = 0.0;

    vec4 aux = texture2D(gaux1, texcoord.st);
    vec3 eye = invproj(vec3(texcoord.st, texture2D(depthtex0, texcoord.st).r));
    vec3 neye = normalize(eye);
    vec4 wpos = gbufferModelViewInverse * vec4(eye, 1.0);
    float dist = length(eye);
    vec3 normal = texture2D(gnormal, texcoord.st).rgb * 2.0 - 1.0;
    vec3 nnormal = normalize(normal);
    float isEyeNotInWater = float(isEyeInWater < 0.5);
    float no_hand = float(aux.g < 0.35 || 0.45 < aux.g);
    
#ifdef CLOUD_MODE
    vec3 cloudColor = mix(ambient_color, gl_Fog.color.rgb, rainStrength * 0.5);
    float cloudIntensity = mix(0.7, 1.0, 1.0 - timeMoon); // Reduce intensity at night
    cloudIntensity = mix(cloudIntensity, 0.85, timeLightTransition); // Adjust for dawn/dusk
    color += cloudRayMarching(cameraPosition, cameraPosition + wpos.xyz + vec3(0.0, 20.0, 0.0)) * cloudColor * (no_hand * 0.5 + 0.2) * cloudIntensity;
#else
    vec3 cloudColor = mix(ambient_color + vec3(0.2), gl_Fog.color.rgb, 0.5 + rainStrength * 0.3);
    float cloudOpacity = mix(0.6, 1.0, 1.0 - timeMoon); // Reduce opacity at night
    cloudOpacity = mix(cloudOpacity, 0.8, timeLightTransition); // Adjust for dawn/dusk
    color = mix(color, cloudColor, cloudRayMarching(cameraPosition, cameraPosition + wpos.xyz + vec3(0.0, 20.0, 0.0)) * (no_hand * 0.5 + 0.2) * cloudOpacity);
#endif
#endif
    // Add fog based on rain
    float fogDensity = mix(0.0, 0.01, rainStrength);
    float fogFactor = 1.0 - exp(-fogDensity * dist);
    color = mix(color, gl_Fog.color.rgb, fogFactor * isEyeNotInWater);

    
    // terrains
    if (0.15 < aux.g) {
        vec3 total_effect = mix(vec3(0.1), ambient_color, 0.22 + (1.0 - no_hand) * 0.58); // sky and indoor ambient light
        
#ifdef DYNAMIC_AMBIENT
        float ambientSample = 0.0;
        vec2 pwph = vec2(12.0) / vec2(viewWidth, viewHeight);   // adjustment to sample range
        for (int i = -4; i < 5; ++i) {
            for (int j = -4; j < 5; ++j) {
                float ambient_aux = texture2D(gaux1, clamp(texcoord.st + pwph * vec2(float(i), float(j)), 0.0, 1.0)).g;
                ambientSample += float(ambient_aux < 0.35);
            }
        }
        ambientSample *= 0.004; // adjustment to ambient mix
        total_effect += mix(vec3(0.0), sunlight_color, ambientSample) * transition_fading * (no_hand * 0.8 + 0.2);  // reflected illumination - reduce effect on hand
#endif
        
        vec3 shading = vec3(1.0);

        float NdotL = dot(nnormal, lightVector);
        float translucent = float(0.25 < aux.g && aux.g < 0.35);  // use lit(0.3)
        float sss_transparency = 0.75 * translucent; //subsurface scattering amount
        float sss = pow(max(dot(neye, lightVector), 0.00001), 20.0) * sss_transparency * NdotL * -4.0;  // NdotL now uses nnormal
        float distof2 = clamp(1.0 - dist / (shadowDistance * 0.75), 0.0, 1.0);
        float sss_fade = pow(distof2, 0.2);
        sss = mix(0.0, sss, min(sss_fade + 0.5, 1.0));
        total_effect += sss * sunlight_color * shading * (1.0 - rainStrength * 0.9) * ambient_transition; // sub surface scattering light
        
        float shadow_fade = clamp(12.0 * (1.0 - dist / shadowDistance), 0.0, 1.0);
        float sunlight_direct = clamp(mix(NdotL, 0.75, translucent), 0.00001, (no_hand * 0.75 + 0.25));
        total_effect += sunlight_color * (1.0 - wetness * 0.1 - rainStrength * 0.9) * mix(vec3(1.0), shading, shadow_fade) * transition_fading * sunlight_direct;   // direct sun light
        
        total_effect += max(dot(nnormal, -neye), 0.00001) * heldLight.rgb * heldLight.a / dist;    // held light
        
        // spectral reflection on water and glasses - for sun and moon only
        spec += mix(nspec(-neye, lightOffset0, nnormal) + nspec(-neye, lightOffset3, nnormal), nspec(-neye, lightVector, nnormal), float(0.999 < transition_fading)) * (1.0 - rainStrength) * float(0.85 < aux.g);
        
        // blocks specular include glasses
        float rspec = max(0.0, dot(reflect(neye, nnormal), lightVector));
        vec4 specularity = texture2D(gdepth, texcoord.st);
        total_effect += (1.0 - rainStrength) * rspec * specularity.rgb * shading * no_hand * transition_fading * float(aux.g < 0.95);

        // torch light and beacon
        float lum = (0.299 * ambient_color.r) + (0.587 * ambient_color.g) + (0.114 * ambient_color.b);
        vec3 litcolor = mix(torchcolor, vec3(lum), float(0.45 < aux.g && aux.g < 0.55));    // beacon: air(0.5)
        total_effect += pow(aux.b, TORCH_POWER) * TORCH_INTENSITY * litcolor * (no_hand * 0.5 + 0.5);   // reduce effect of torch on hand
        color *= mix(vec3(1.0), total_effect, isEyeNotInWater);
        
        // lit up torches
        float fire_lit = float(0.17 < texture2D(gaux1, texcoord.st).r) * flicker * 3.0;
        color += clamp((texture2D(gcolor, texcoord.st).rgb - vec3(0.5)) * fire_lit, 0.0, 1.0);
        
        // smokes and particles
        color = mix(color, texture2D(gcolor, texcoord.st).rgb, float(aux.a < 0.5) * 0.15);
    }
    else {
        // sky and raining
        float rainCondition = rainStrength * 0.1 * isEyeNotInWater;
        color = mix(color, gl_Fog.color.rgb, rainCondition);
    }

    
    // eye in water
    vec3 watercolor = vec3(0.0431, 0.0529, 0.0627) * (ambient_color.r + ambient_color.g + ambient_color.b) * 5.5;
    float log_depth = ld(texture2D(depthtex2, texcoord.st).r);  
    color = mix(color, vec3(1.0), log_depth * log_depth * isEyeInWater * 1.2); // Increase fog by multiplying with 1.2
    color *= mix(vec3(1.0), watercolor, isEyeInWater);
	



    
    color += texture2D(gaux2, texcoord.xy).rgb * texture2D(gaux2, texcoord.xy).a * 0.5 * (no_hand * 0.7 + 0.3) * isEyeNotInWater;	// draw rain

    color = clamp(color, 0.0, 1.0);

/* DRAWBUFFERS:0 */
    gl_FragData[0] = vec4(color, spec);	// gcolor in composite1.fsh
}

//////////////////////////////main//////////////////////////////
