#version 120

uniform mat4 gbufferProjection;
uniform mat4 gbufferModelView;

uniform vec3 sunPosition;
uniform vec3 moonPosition;

uniform float rainStrength;
uniform float frameTimeCounter;

uniform int worldTime;
uniform int heldItemId;

varying vec4 texcoord;
varying vec4 heldLight;

varying vec3 lightVector;
varying vec3 lightOffset0;
varying vec3 lightOffset3;
varying vec3 sunlight_color;
varying vec3 ambient_color;

varying float transition_fading;
varying float ambient_transition;
varying float gp00;
varying float gp11;
varying float gp22;
varying float gp32;
varying float gp2232;
varying vec2 igp;
varying float flicker;

//////////////////////////////main//////////////////////////////

void main() {
    gl_Position = gl_ProjectionMatrix * gl_ModelViewMatrix * gl_Vertex;
    
    texcoord = gl_MultiTexCoord0;

    float wtime = float(worldTime);
    transition_fading = 1.0 - (clamp((wtime - 12100.0) / 300.0, 0.0, 1.0) - clamp((wtime - 13200.0) / 300.0, 0.0, 1.0) + clamp((wtime - 22500.0) / 300.0, 0.0, 1.0) - clamp((wtime - 23600.0) / 300.0, 0.0, 1.0));
    ambient_transition = 1.0 - (clamp((wtime - 12550.0) / 250.0, 0.0, 1.0) - clamp((wtime - 12800.0) / 250.0, 0.0, 1.0) + clamp((wtime - 22950.0) / 250.0, 0.0, 1.0) - clamp((wtime - 23200.0) / 250.0, 0.0, 1.0));
    
    float isDay = float(worldTime < 12800 || 23200 < worldTime);
    lightVector = normalize(mix(moonPosition, sunPosition, isDay));

    float sunlight_radius = mix(0.06, 0.1, isDay) * (1.0 - transition_fading);
    lightOffset0 = normalize(lightVector + (gbufferModelView * vec4(0.0, sunlight_radius, 0.0, 0.0)).xyz);

    float moonlight_radius = mix(0.1, 0.06, isDay) * (1.0 - transition_fading);
    lightOffset3 = normalize(-lightVector + (gbufferModelView * vec4(0.0, moonlight_radius, 0.0, 0.0)).xyz);
    
    flicker = (sin(3.142 * (sin(4.935 * frameTimeCounter) + sin(1.571 * frameTimeCounter))) * 0.05 + 0.95);
    float heldFlicker = cos(-3.142 * (sin(4.935 * frameTimeCounter) + sin(1.885 * frameTimeCounter))) * 0.06 + 0.54;

    heldLight = float(heldItemId == 50) * vec4(0.9, 0.7, 0.4, heldFlicker);   // torch
    heldLight += float(heldItemId == 76 || heldItemId == 94) * vec4(1.0, 0.2, 0.2, 0.5 * heldFlicker);  // active redstone torch / redstone repeater
    heldLight += float(heldItemId == 89) * vec4(0.9, 0.9, 0.5, 0.5);    // lightstone
    heldLight += float(heldItemId == 91) * vec4(0.7, 0.5, 0.3, 0.7 * heldFlicker);  // jack-o-lantern
    heldLight += float(heldItemId == 327) * vec4(0.9, 0.5, 0.3, 0.3 * heldFlicker); // lava bucket
    heldLight += vec4(-0.075);    // glowing self

    float hour = wtime / 1000.0 + 6.0;
    hour += -24.0 * float(24.0 < hour);
    
    //////////////////////rain color//////////////////////
    float rain_color = 0.4 - cos(hour / 12.0 * 3.141597) * 0.3;
    
    ////////////////////sunlight color////////////////////
    const ivec4 ToD[25] = ivec4[25](ivec4(0,15,30,70), //hour,r,g,b
                                    ivec4(1,15,30,70),
                                    ivec4(2,15,30,70),
                                    ivec4(3,15,30,70),
                                    ivec4(4,15,30,70),
                                    ivec4(5,50,60,80),
                                    ivec4(6,245,190,70),
                                    ivec4(7,255,195,80),
                                    ivec4(8,255,200,97),
                                    ivec4(9,255,210,110),
                                    ivec4(10,255,215,125),
                                    ivec4(11,255,225,140),
                                    ivec4(12,255,235,150),
                                    ivec4(13,255,225,140),
                                    ivec4(14,255,215,125),
                                    ivec4(15,255,210,110),
                                    ivec4(16,255,200,97),
                                    ivec4(17,255,195,80),
                                    ivec4(18,245,190,70),
                                    ivec4(19,77,67,194),
                                    ivec4(20,15,30,70),
                                    ivec4(21,15,30,70),
                                    ivec4(22,15,30,70),
                                    ivec4(23,15,30,70),
                                    ivec4(24,15,30,70));
    ivec4 temp = ToD[int(floor(hour))];
    ivec4 temp2 = ToD[int(floor(hour)) + 1];
    
    sunlight_color = mix(vec3(temp.yzw), vec3(temp2.yzw), (hour - float(temp.x)) / float(temp2.x - temp.x)) / 255.0;
    sunlight_color = mix(sunlight_color, vec3(rain_color), rainStrength * 0.8);
    
    ////////////////////ambient color////////////////////
    const ivec4 ToD2[25] = ivec4[25](ivec4(0,50,80,140), //hour,r,g,b
                                     ivec4(1,50,80,140),
                                     ivec4(2,50,80,140),
                                     ivec4(3,60,90,150),
                                     ivec4(4,60,90,150),
                                     ivec4(5,75,106,155),
                                     ivec4(6,160,170,255),
                                     ivec4(7,160,175,255),
                                     ivec4(8,160,180,260),
                                     ivec4(9,165,190,270),
                                     ivec4(10,190,205,280),
                                     ivec4(11,205,230,290),
                                     ivec4(12,220,255,300),
                                     ivec4(13,205,230,290),
                                     ivec4(14,190,205,280),
                                     ivec4(15,165,190,270),
                                     ivec4(16,150,176,260),
                                     ivec4(17,140,160,255),
                                     ivec4(18,128,150,255),
                                     ivec4(19,77,67,194),
                                     ivec4(20,60,90,150),
                                     ivec4(21,60,90,150),
                                     ivec4(22,50,80,140),
                                     ivec4(23,50,80,140),
                                     ivec4(24,50,80,140));
    ivec4 tempa = ToD2[int(floor(hour))];
    ivec4 tempa2 = ToD2[int(floor(hour)) + 1];
    
    ambient_color = mix(vec3(tempa.yzw), vec3(tempa2.yzw), (hour - float(tempa.x)) / float(tempa2.x - tempa.x)) / 255.0;
    ambient_color = mix(ambient_color, vec3(rain_color), rainStrength * 0.5);

    gp00 = gbufferProjection[0][0] * -0.5;
    gp11 = gbufferProjection[1][1] * -0.5;
    gp22 = gbufferProjection[2][2] * 0.5;
    gp32 = gbufferProjection[3][2] * 0.5;
    gp2232 = -gp22 - gp32;
    igp = vec2(-1.0) / vec2(gp00, gp11);
}




//////////////////////////////main//////////////////////////////
