#version 120

uniform sampler2D gcolor;
uniform sampler2D gdepth;
uniform sampler2D gnormal;
uniform sampler2D lightmap;
uniform float viewHeight;
uniform float viewWidth;
uniform float frameTimeCounter;

varying vec2 texcoord;

const float snowIntensity = 5.5; // Adjust this value to control snow intensity (0.0 to 1.0)

// Function to generate noise
float noise(vec2 coord) {
    return fract(sin(dot(coord, vec2(12.9898, 78.233))) * 43758.5453);
}

void main() {
    // Sample the original color
    vec4 color = texture2D(gcolor, texcoord);
    
    // Sample the lightmap
    vec4 light = texture2D(lightmap, texcoord);
    
    // Sample the normal map
    vec3 normal = normalize(texture2D(gnormal, texcoord).rgb * 2.0 - 1.0);
    
    // Calculate snow effect
    float snowNoise = noise(texcoord * vec2(viewWidth, viewHeight) / 32.0 + frameTimeCounter * 0.1);
    float snowFactor = max(0.0, normal.y); // More snow on upward-facing surfaces
    float snow = snowNoise * snowFactor * snowIntensity;
    
    // Combine color, light, and snow
    vec3 finalColor = mix(color.rgb * light.rgb, vec3(1.0), snow);
    
    // Apply some basic enhancements
    finalColor = pow(finalColor, vec3(1.0 / 2.2)); // Gamma correction
    finalColor *= 1.1; // Slight brightness boost
    
    // Output the final color
    gl_FragData[0] = vec4(finalColor, 1.0);
}

