#version 120
#include "/settings/lib.h"

uniform sampler2D texture;
uniform sampler2D lightmap;
uniform sampler2D noisetex;
uniform int isEyeInWater;
uniform float far;
uniform vec3 fogColor;
uniform float frameTimeCounter;
uniform vec3 skyColor;
uniform float rainStrength;

varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 normal;
varying vec4 glcolor;
varying float isWater;
varying vec4 position;
varying vec4 worldPosition;
varying vec2 waterTexCoord;

#define Fogend 0.5
#define fogstart 0.2

const float fogStart = 0.2;
const float fogEnd = 0.5;

void main() {
    vec4 color = texture2D(texture, texcoord) * glcolor;
    vec4 light = texture2D(lightmap, lmcoord);
    
    color.rgb *= light.rgb;
    
    if (isWater > 0.5) {
        vec3 waterColor = vec3(0.1137, 0.1686, 0.2275);
        
        // Enhance noise effect when raining
        float noiseScale = 1.0 + rainStrength * 1.5; // Increase noise scale with rain
        float noiseSpeed = 1.0 + rainStrength * 1.5; // Increase noise speed with rain
        
        vec2 nCoord = waterTexCoord * noiseScale + vec2(frameTimeCounter * 0.03 * noiseSpeed, frameTimeCounter * 0.02 * noiseSpeed);
        float noise1 = texture2D(noisetex, nCoord).r;
        float noise2 = texture2D(noisetex, nCoord * 1.4 + vec2(frameTimeCounter * 0.02 * noiseSpeed, -frameTimeCounter * 0.015 * noiseSpeed)).r;
        float finalNoise = (noise1 + noise2) * 0.9;
        
        // Apply enhanced noise to water color
        color.rgb = mix(waterColor, waterColor * (1.0 + finalNoise * (0.2 + rainStrength * 0.1)), 0.5 + rainStrength * 0.2);
        color.a = 0.7 - rainStrength * 0.1; // Slightly reduce opacity when raining
    }
    
    // Apply fog
    float dist = length(position.xyz);
    float fogFactor = clamp((dist - far * fogStart) / (far * (fogEnd - fogStart)), 0.0, 1.0);
    vec3 fogColorFinal = isEyeInWater == 1 ? vec3(0.0, 0.1, 0.2) : fogColor;
    
    color.rgb = mix(color.rgb, fogColorFinal, fogFactor);
    
    gl_FragData[0] = color;
}
