#version 120
#include "/settings/lib.h"

attribute vec4 mc_Entity;
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjection;
uniform float frameTimeCounter;
uniform vec3 cameraPosition;
uniform float rainStrength;

varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 normal;
varying vec4 glcolor;
varying float isWater;
varying vec4 position;
varying vec4 worldPosition;
varying vec2 waterTexCoord;

void main() {
    gl_Position = ftransform();
    
    texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
    lmcoord = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
    glcolor = gl_Color;
    normal = gl_NormalMatrix * gl_Normal;
    
    isWater = 0.0;
    if (mc_Entity.x == 8.0 || mc_Entity.x == 9.0) {
        isWater = 1.0;
        
        // Enhance wave effect when raining
        float waveIntensity = 0.05 + rainStrength * 0.1; // Increase wave intensity with rain
        float waveSpeed = 2.0 + rainStrength * 1.5; // Increase wave speed with rain
        
        // Apply vertical displacement
        worldPosition.y += sin(frameTimeCounter * waveSpeed + worldPosition.x * 0.5 + worldPosition.z * 0.5) * waveIntensity;
    }
    
    position = gbufferModelView * gl_Vertex;
    worldPosition = gl_Vertex;
    waterTexCoord = worldPosition.xz / 40.0;
    
    gl_Position = gl_ProjectionMatrix * gbufferModelView * worldPosition;
}
