#version 120

varying vec4 texcoord;
varying vec3 sunlight_color;
varying vec3 ambient_color;
varying float isEyeInWater;

uniform int worldTime;
uniform float rainStrength;
uniform vec3 sunPosition;
uniform vec3 moonPosition;

void main() {
    gl_Position = ftransform();
    texcoord = gl_MultiTexCoord0;

    // Time of day calculations
    float hour = float(worldTime) / 1000.0 + 6.0;
    if (hour > 24.0) hour -= 24.0;

    // Sunlight color calculation
    vec3 dayColor = vec3(0.8275, 0.7569, 0.7569);
    vec3 nightColor = vec3(0.2, 0.3, 0.4);
    sunlight_color = mix(nightColor, dayColor, smoothstep(4.0, 8.0, hour) - smoothstep(16.0, 20.0, hour));

    // Ambient color calculation
    vec3 dayAmbient = vec3(0.6, 0.7, 1.0);
    vec3 nightAmbient = vec3(0.1, 0.15, 0.2);
    ambient_color = mix(nightAmbient, dayAmbient, smoothstep(4.0, 8.0, hour) - smoothstep(16.0, 20.0, hour));

    // Apply rain effect
    sunlight_color = mix(sunlight_color, vec3(0.7), rainStrength);
    ambient_color = mix(ambient_color, vec3(0.5), rainStrength);

    // Determine if the player is underwater
    isEyeInWater = float(gl_ModelViewMatrix[3][2] > 1.0);
}