#version 120

uniform sampler2D gcolor;
uniform float far;
uniform int isEyeInWater;
uniform vec3 fogColor;

varying vec2 texcoord;
varying vec4 position;

const float fogStart = 0.2;
const float fogEnd = 0.5;

void main() {
    vec4 color = texture2D(gcolor, texcoord);

    // Fog calculation
    float dist = length(position.xyz);
    float fogFactor = clamp((dist - far * fogStart) / (far * (fogEnd - fogStart)), 0.0, 1.0);
    
    // Define white fog color
    vec3 whiteFogColor = vec3(1.0, 1.0, 1.0);
    
    // Choose fog color based on conditions
    vec3 fogColorFinal;
    if (isEyeInWater == 1) {
        fogColorFinal = vec3(0.0, 0.1, 0.2); // Underwater fog color
    } else {
        fogColorFinal = mix(fogColor, whiteFogColor, 0.7); // Mix with white fog
    }

    // Apply fog
    color.rgb = mix(color.rgb, fogColorFinal, fogFactor);

    gl_FragColor = color;
}
